package org.openxma.dsl.generator.helper;

import org.eclipse.emf.ecore.EObject;
import org.openxma.dsl.core.model.AdditiveExpr;
import org.openxma.dsl.core.model.BoolLiteral;
import org.openxma.dsl.core.model.Call;
import org.openxma.dsl.core.model.CondANDExpr;
import org.openxma.dsl.core.model.CondORExpr;
import org.openxma.dsl.core.model.EqualityExpr;
import org.openxma.dsl.core.model.EqualityOp;
import org.openxma.dsl.core.model.Function;
import org.openxma.dsl.core.model.IntLiteral;
import org.openxma.dsl.core.model.MultiplicativeExpr;
import org.openxma.dsl.core.model.ParenExpr;
import org.openxma.dsl.core.model.ReferenceableByXmadslVariable;
import org.openxma.dsl.core.model.RelationalExpr;
import org.openxma.dsl.core.model.SimpleType;
import org.openxma.dsl.core.model.StatusFlag;
import org.openxma.dsl.core.model.StringLiteral;
import org.openxma.dsl.core.model.XmadslVariable;
import org.openxma.dsl.core.util.CoreSwitch;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.dom.model.ComplexType;
import org.openxma.dsl.dom.model.Property;

/**
 * Switch to evaluate the type name of a given Expression.
 */
public class ExpressionTypeSwitch extends CoreSwitch<String> {
	@Override
	public String caseEqualityExpr(EqualityExpr equalityExpr) {
		if (EqualityOp.NULL.equals(equalityExpr.getOp())) {
			return doSwitch(equalityExpr.getLeft());
		}
		return Boolean.class.getSimpleName();
	}

	@Override
	public String caseCondORExpr(CondORExpr condORExpr) {
		if (condORExpr.getRights().isEmpty()) {
			return doSwitch(condORExpr.getLeft());
		}
		return Boolean.class.getSimpleName();
	}

	@Override
	public String caseCondANDExpr(CondANDExpr condANDExpr) {
		if (condANDExpr.getRights().isEmpty()) {
			return doSwitch(condANDExpr.getLeft());
		}
		return Boolean.class.getSimpleName();
	}

	@Override
	public String caseRelationalExpr(RelationalExpr relationalExpr) {
		return doSwitch(relationalExpr.getLeft());
	}

	@Override
	public String caseAdditiveExpr(AdditiveExpr additiveExpr) {
		return doSwitch(additiveExpr.getLeft());
	}

	@Override
	public String caseMultiplicativeExpr(MultiplicativeExpr multiplicativeExpr) {
		return doSwitch(multiplicativeExpr.getLeft());
	}

	@Override
	public String caseXmadslVariable(XmadslVariable xmadslVariable) {
		ReferenceableByXmadslVariable referenceableByXmadslVariable = xmadslVariable
				.getReference();
		if (referenceableByXmadslVariable instanceof Property) {
			Property property = (Property) referenceableByXmadslVariable;
			return doSwitch(property.getType());
		} else if (referenceableByXmadslVariable instanceof Attribute) {
			Attribute attribute = (Attribute) referenceableByXmadslVariable;
			doSwitch(attribute.getType().getDataType());
		}
		return doSwitch(referenceableByXmadslVariable);
	}

	@Override
	public String caseStatusFlag(StatusFlag statusFlag) {
		return Boolean.class.getSimpleName();
	}

	@Override
	public String caseIntLiteral(IntLiteral object) {
		return Integer.class.getSimpleName();
	}

	@Override
	public String caseBoolLiteral(BoolLiteral object) {
		return Boolean.class.getSimpleName();
	}

	@Override
	public String caseStringLiteral(StringLiteral object) {
		return String.class.getSimpleName();
	}

	@Override
	public String caseParenExpr(ParenExpr parenExpression) {
		return doSwitch(parenExpression.getExpr());
	}

	@Override
	public String caseCall(Call call) {
		return doSwitch(call.getFunction());
	}

	@Override
	public String caseFunction(Function function) {
		return doSwitch(function.getResultType());
	}

	@Override
	public String caseSimpleType(SimpleType object) {
		return SimpleTypeExtension.getJavaType(object);
	}

	@Override
	public String defaultCase(EObject object) {
		if (object instanceof ComplexType) {
			return ((ComplexType) object).getName();
		}
		return super.defaultCase(object);
	}

}
