package org.openxma.dsl.generator.helper;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.xtext.EcoreUtil2;
import org.openxma.dsl.common.extensions.EObjectPropertiesAdapter;
import org.openxma.dsl.core.CoreFactory;
import org.openxma.dsl.core.model.BoolLiteral;
import org.openxma.dsl.core.model.EqualityExpr;
import org.openxma.dsl.core.model.Style;
import org.openxma.dsl.generator.GeneratorConfiguration;
import org.openxma.dsl.generator.helper.style.ApplyCombinedStyles;
import org.openxma.dsl.pom.PomFactory;
import org.openxma.dsl.pom.PomPackage;
import org.openxma.dsl.pom.model.CollapsedFlag;
import org.openxma.dsl.pom.model.Component;
import org.openxma.dsl.pom.model.Composite;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.EditableFlag;
import org.openxma.dsl.pom.model.EnabledFlag;
import org.openxma.dsl.pom.model.FieldFlag;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.IElementWithLayoutData;
import org.openxma.dsl.pom.model.ImageProperty;
import org.openxma.dsl.pom.model.ImageUri;
import org.openxma.dsl.pom.model.ModelElementSpecification;
import org.openxma.dsl.pom.model.PaddingWidth;
import org.openxma.dsl.pom.model.PageProperty;
import org.openxma.dsl.pom.model.StyleProperty;
import org.openxma.dsl.pom.model.StylePropertyAlignment;
import org.openxma.dsl.pom.model.StylePropertyBackgroundColor;
import org.openxma.dsl.pom.model.StylePropertyEnumerationDisplayType;
import org.openxma.dsl.pom.model.StylePropertyEnumerationSortOrder;
import org.openxma.dsl.pom.model.StylePropertyFieldPart;
import org.openxma.dsl.pom.model.StylePropertyFlag;
import org.openxma.dsl.pom.model.StylePropertyFont;
import org.openxma.dsl.pom.model.StylePropertyForegroundColor;
import org.openxma.dsl.pom.model.StylePropertyImage;
import org.openxma.dsl.pom.model.StylePropertyLineWrap;
import org.openxma.dsl.pom.model.StylePropertyMultiLine;
import org.openxma.dsl.pom.model.StylePropertyOther;
import org.openxma.dsl.pom.model.StylePropertyTable;
import org.openxma.dsl.pom.model.StylePropertyTabulator;
import org.openxma.dsl.pom.model.StylePropertyWidgetVariant;
import org.openxma.dsl.pom.model.StyleSpecification;
import org.openxma.dsl.pom.model.StyleSpecificationProperty;
import org.openxma.dsl.pom.model.TabPage;
import org.openxma.dsl.pom.model.TableColumn;
import org.openxma.dsl.pom.model.VisibleFlag;
import org.openxma.dsl.pom.model.XmadslPage;
import org.openxma.dsl.pom.naming.PomNameProvider;
import org.openxma.dsl.pom.scoping.PomDslScopeProvider;
import org.openxma.dsl.pom.util.LayoutData;
import org.openxma.dsl.pom.util.StyleAccess;

public class PresentationModelExtension {

    private static final String STYLE_PROPERTY_NAME = "styles";
    private static final String FLAG_FROM_STYLE = "flagFromStyle";    

    public static void setFlagFromStyle(EObject obj) {
        if (obj != null) {
            EObjectPropertiesAdapter.setProperty(obj,FLAG_FROM_STYLE,new Boolean(true));
        }
    }

    public static boolean isFlagFromStyle(EObject obj) {
        if (obj != null) {
            Object info = EObjectPropertiesAdapter.getProperty(obj,FLAG_FROM_STYLE);
            if (info != null && info instanceof Boolean && ((Boolean)info).booleanValue()) {
                return true;
            }
        }
        return false;
    }

    public static void extendPresentationModel(Component comp) {

        if (comp != null) {
            try {
                setParentExpressionsToChild(comp);
                determineDefaultStyles(comp); // TODO remove this call and the according implementation behind
                setElementStyleProperties(comp);
                extendModelElementsForEmptyStyle(comp);
                extendModelElementsForStylesProperties(comp);
            } catch (Exception ex) {
                ex.printStackTrace();
                throw new RuntimeException("Can not extend presentation model",ex);
            }
        }
    }

    static private boolean isBranchToSkip(EObject modelElement) {

        if (modelElement instanceof XmadslPage) {
            XmadslPage xp = (XmadslPage)modelElement;
            if (xp.isAbstract()) return true;
        }
        return false;
    }


    public static void setStyleProperty(EObject obj, List<Style> list) {
        List<Style> filteredStyles = new ArrayList<Style>();
        List<Style> flatStyles = flattenStyles(list);
        for (Style styleToSet : flatStyles) {
            if (shouldStyleBeApplied(obj, styleToSet)) {
                filteredStyles.add(styleToSet);
            }
        }
        if (filteredStyles.size()>0) {
            Map<Integer,StyleSpecificationProperty> combinedStyleSpecificationProperties = combineStyles(filteredStyles);
            if (combinedStyleSpecificationProperties != null) {
                StyleAccess.setCombinedStyleProperty(obj, combinedStyleSpecificationProperties);
            }
            EObjectPropertiesAdapter.setProperty(obj,STYLE_PROPERTY_NAME,filteredStyles);
        }
    }

    public static Map<Integer,StyleSpecificationProperty> combineStyles(List<Style> list) {
        if (list == null || list.size() == 0) {
            return null;
        }
        Map<Integer,StyleSpecificationProperty> propertyMap = new HashMap<Integer,StyleSpecificationProperty>();
        for (Style style : list) {
            if (style instanceof StyleSpecification) {
                StyleSpecification styleSpec = (StyleSpecification)style;
                for (StyleSpecificationProperty specProperty : styleSpec.getStyleProperty()) {
                    int classifierId = specProperty.eClass().getClassifierID();
                    StyleSpecificationProperty existingStyleSpec = propertyMap.get(classifierId);
                    if (existingStyleSpec != null && haveToCombineStyleSpecification(existingStyleSpec)) {
                        // Use a clone for the combined StyleSpecificationProperty in order to avoid a modification
                        // of the original model element which may be used somewhere else and could produce side effects
                        // (The container check is only true the first time because we do not insert the clone into a container.)
                        if (existingStyleSpec.eContainer() != null) {
                            existingStyleSpec = (StyleSpecificationProperty)EcoreUtil2.clone(existingStyleSpec);
                            propertyMap.put(classifierId,existingStyleSpec);                            
                        }                             
                        // combine the properties (overriding is not enough)
                        combineStyleSpecification(existingStyleSpec,specProperty);
                    } else {
                        // simple override by putting it into the map
                        propertyMap.put(classifierId,specProperty);
                    }
                }
            }
        }
        if (propertyMap.values().size()>0) {
            return propertyMap;
        } else {
            return null;
        }
    }
    
    private static boolean haveToCombineStyleSpecification(StyleSpecificationProperty existingStyleSpec) {
        /* StylePropertyOther */
        if (existingStyleSpec instanceof StylePropertyOther) {
            return true;
        }
        /* StylePropertyFlag */              
        if (existingStyleSpec instanceof StylePropertyFlag) {
            return true;
        }            
        /* StylePropertyTable */            
        if (existingStyleSpec instanceof StylePropertyTable) {
            return true;
        }                  
        return false;                   
    }

    private static void combineStyleSpecification(StyleSpecificationProperty existingStyleSpec, StyleSpecificationProperty specProperty) {        
        /* StylePropertyOther */
        if (existingStyleSpec instanceof StylePropertyOther) {
            StylePropertyOther existing = (StylePropertyOther)existingStyleSpec;
            StylePropertyOther spec = (StylePropertyOther)specProperty;
            if (spec.getDynamicTabPage() != null) {
                existing.setDynamicTabPage((BoolLiteral)EcoreUtil2.clone(spec.getDynamicTabPage()));
            }
            if (spec.getMargin() != null) {
                existing.setMargin((PaddingWidth)EcoreUtil2.clone(spec.getMargin()));
            }
            if (spec.getSpacing() != null) {
                existing.setSpacing((PaddingWidth)EcoreUtil2.clone(spec.getSpacing()));
            }
            if (spec.getThousandSeperator() != null) {
                existing.setThousandSeperator((BoolLiteral)EcoreUtil2.clone(spec.getThousandSeperator()));
            }
            if (spec.getWidgetEffects().size()>0) {
                existing.getWidgetEffects().addAll(spec.getWidgetEffects());
            }            
        /* StylePropertyFlag */              
        } else if (existingStyleSpec instanceof StylePropertyFlag) {
            StylePropertyFlag existing = (StylePropertyFlag)existingStyleSpec;
            StylePropertyFlag spec = (StylePropertyFlag)specProperty;  
            if (spec.getEnabledExpression() != null) {
                existing.setEnabledExpression((EqualityExpr)EcoreUtil2.clone(spec.getEnabledExpression()));                
            }
            if (spec.getMandatoryExpression() != null) {
                existing.setMandatoryExpression((EqualityExpr)EcoreUtil2.clone(spec.getMandatoryExpression()));                
            }
            if (spec.getVisibleExpression() != null) {
                existing.setVisibleExpression((EqualityExpr)EcoreUtil2.clone(spec.getVisibleExpression()));                
            }
            if (spec.getCollapsedExpression() != null) {
                existing.setCollapsedExpression((EqualityExpr)EcoreUtil2.clone(spec.getCollapsedExpression()));                
            }
            if (spec.getAvailableExpression() != null) {
                existing.setAvailableExpression((EqualityExpr)EcoreUtil2.clone(spec.getAvailableExpression()));                
            }
            if (spec.getEditableExpression() != null) {
                existing.setEditableExpression((EqualityExpr)EcoreUtil2.clone(spec.getEditableExpression()));                
            }               
            if (spec.getTabableExpression() != null) {
                existing.setTabableExpression((EqualityExpr)EcoreUtil2.clone(spec.getTabableExpression()));                
            }               
        /* StylePropertyTable */            
        } else if (existingStyleSpec instanceof StylePropertyTable) {
            StylePropertyTable existing = (StylePropertyTable)existingStyleSpec;
            StylePropertyTable spec = (StylePropertyTable)specProperty;
            if (spec.getGridVisible() != null) {
                existing.setGridVisible((BoolLiteral)EcoreUtil2.clone(spec.getGridVisible()));
            }
            if (spec.getOneWay() != null) {
                existing.setOneWay((BoolLiteral)EcoreUtil2.clone(spec.getOneWay()));
            }
        /* Combination not supported */
        } else {                  
            throw new RuntimeException("Combination of StyleSpecificationProperty *"+existingStyleSpec.getClass().getName()+"' not supported");
        }
    }

    public static boolean shouldStyleBeApplied(EObject obj, Style styleToSet) {

        /* Style has no selector */
        if (styleToSet instanceof StyleSpecification == false) return true;

        /* Style may have selectors */
        List<ModelElementSpecification> mesList = ((StyleSpecification)styleToSet).getModelElements();
        if (mesList.size()>0) {
            for (ModelElementSpecification mes : mesList) {
                if (StylePropertyExtension.fitsSelector(obj,mes)) {
                    return true;
                }
            }
        } else {
            return true;
        }
        return false;
    }

    /**
     * Set the FieldFlags defined in a container to its children (join the flags for every child)
     *
     * @param modelElement
     */
    private static void setParentExpressionsToChild(EObject modelElement) {
        if (isBranchToSkip(modelElement)) return;
        Iterator<EObject> contentIterator = modelElement.eContents().iterator();
        while (contentIterator.hasNext()) {
            EObject obj = contentIterator.next();
            List<FieldFlag> fieldFlagsOfContainer = FieldFlags.getFieldFlagsOfContainer(obj);
            if (fieldFlagsOfContainer != null) {            
                ArrayList<FieldFlag> copy = new ArrayList<FieldFlag>();
                copy.addAll(fieldFlagsOfContainer);                
                if (copy.size()>0) {
                    for (int i=copy.size()-1;i>=0;i--) {
                        FieldFlag f = copy.get(i);
                        if (f instanceof VisibleFlag) {
                            copy.remove(f);
                        }
                    }                    
                }
                if (copy.size()>0) {
                List<EObject> childGuiElements = PomDslScopeProvider.getContainedGuiElements(obj);
                if (childGuiElements != null) {
                    for (EObject child : childGuiElements) {
                            FieldFlags.joinFieldFlags(child, copy);
                        }
                    }
                }
            }
            setParentExpressionsToChild(obj);
        }
    }

    private static void determineDefaultStyles(Component xmadslComponent) {
        TreeIterator<EObject> allContents = xmadslComponent.eAllContents();
        while (allContents.hasNext()) {
            EObject obj = allContents.next();
            if (isBranchToSkip(obj)) continue;
            if (canModelElementHaveStyle(obj)) {
                determineDefaultStyleForModelElement(obj);
            }
        }
    }

    private static void extendModelElementsForEmptyStyle(EObject xmaComp) {
        TreeIterator<EObject> allContents = xmaComp.eAllContents();
        while (allContents.hasNext()) {
            EObject obj = allContents.next();
            if (isBranchToSkip(obj)) continue;
            extendModelElement(obj,null);
        }
    }

    private static void extendModelElementsForStylesProperties(EObject xmaComp) {
        TreeIterator<EObject> allContents = xmaComp.eAllContents();
        while (allContents.hasNext()) {
            EObject obj = allContents.next();
            if (isBranchToSkip(obj)) continue;
            // First implementation
            // TODO remove/replace implementations in extendModelElement->applyStyleSpecification by implementations in applyCombinedStyle
            List<Style> styles = PresentationModelExtension.getStyleProperty(obj);
            if (styles != null) {
                for (Style style : styles) {
                    extendModelElement(obj,style);
                }
            }
            // Current implementation
            // TODO take over features implemented in first implementation (see above)
            // At this point the combined style is available for each (existing) presentation model
            // element. Therefore we can now apply all styles which are modifying the presentation model itself.
            Map<Integer,StyleSpecificationProperty> styleProperties = StyleAccess.getCombinedStyleProperty(obj);
            if (styleProperties != null) {
                ApplyCombinedStyles.applyToPresentationModelObject(obj,styleProperties);
            }
        }
    }

    public static List<Style> flattenStyles(List<? extends Style> styles) {
        if (styles == null) {
            return null;
        }
        List<Style> stylesIncludingFlatSuperStyles = new ArrayList<Style>();
        for (Style style : styles) {
            if (style instanceof StyleSpecification) {
                StyleSpecification styleSpec = (StyleSpecification)style;
                if (styleSpec.getSuperStyles() != null && styleSpec.getSuperStyles().size()>0) {
                    List<Style> flatSuperStyles = flattenStyles(styleSpec.getSuperStyles());
                    stylesIncludingFlatSuperStyles.addAll(flatSuperStyles);
                }
            }
        }
        stylesIncludingFlatSuperStyles.addAll(styles);
        return stylesIncludingFlatSuperStyles;
    }

    private static void setElementStyleProperties(EObject modelElement) {
        List<Style> styles = getStyles(modelElement);
        if (styles != null) {
            setStyleProperty(modelElement,styles);
        }
        setChildElementStyles(modelElement,styles);
    }

    private static void setChildElementStyles(EObject modelElement, List<Style> parentStyles) {
        if (isBranchToSkip(modelElement)) return;
        List<EObject> allContents = modelElement.eContents();
        if (allContents == null) {
            return;
        }
        for (EObject obj : allContents) {
            if (obj == null) {
                continue;
            }
            List<Style> styles = new ArrayList<Style>();
            if (parentStyles != null) {
                styles.addAll(parentStyles);
            }            
            List<Style> objStyles = getStyles(obj);
            if (objStyles != null) {
                styles.addAll(objStyles);
            }
            if (styles != null) {
                setStyleProperty(obj,styles);
            }
            setChildElementStyles(obj,styles);
        }
    }

    private static void extendModelElement(EObject obj, Style style) {

        if (isBranchToSkip(obj)) return;
        
        if (obj instanceof IElementWithLayoutData) {
            IElementWithLayoutData guiElement = (IElementWithLayoutData)obj;            
            LayoutDataExtension.setDefaultWidth(guiElement,style);
        }

        String styleName = (style != null) ? style.getName():null;
        if (style instanceof StyleSpecification) {
            applyStyleSpecification(obj, (StyleSpecification)style);
        }
        if (obj instanceof Composite) {
            Composite composite = (Composite)obj;
            TabulatorExtension.initTabulator(composite,styleName);
        }

        // Default hook for Xmadsl-model
        GeneratorConfiguration.INSTANCE.getLayoutStrategy().setCustomXmadslModelElementProperties(obj,styleName);

    }

    private static void applyStyleSpecification(EObject obj, StyleSpecification sSpec) {
        FieldFlags fieldFlags = null;
        for (StyleSpecificationProperty sp : sSpec.getStyleProperty()) {
            if (sp instanceof StylePropertyFieldPart && obj instanceof CustomizeableField) {
                CustomizeableField cf = (CustomizeableField)obj;
                StylePropertyExtension.setFieldPartSpecificationFromStyle(sp, cf);
            } else if (obj instanceof IElementWithLayoutData) {
                IElementWithLayoutData eld = (IElementWithLayoutData)obj;
                StylePropertyExtension.setLayoutPropertyFromStyle(sp, eld);
            }
            if (sp instanceof StylePropertyTabulator && obj instanceof GuiElement) {
                GuiElement gel = (GuiElement)obj;
                StylePropertyExtension.setTabulatorFromStyle((StylePropertyTabulator)sp, gel);
            } else if (sp instanceof StylePropertyFlag) {
                StylePropertyFlag spf = (StylePropertyFlag)sp;
                if (obj instanceof IElementWithLayoutData) {
                    IElementWithLayoutData eld = (IElementWithLayoutData)obj;
                    if (fieldFlags == null) fieldFlags = new FieldFlags(eld.getFieldFlags());
                    FieldFlag newFlag = null;
                    if (spf.getEnabledExpression() != null && (fieldFlags.getEnabledFlag() == null || isFlagFromStyle(fieldFlags.getEnabledFlag()))) {
                        newFlag = PomFactory.eINSTANCE.createEnabledFlag();
                        newFlag.setExpression((EqualityExpr)EcoreUtil.copy(spf.getEnabledExpression()));
                        fieldFlags.setEnabledFlag((EnabledFlag)newFlag);
                    } else if (spf.getVisibleExpression() != null && (fieldFlags.getVisibleFlag() == null || isFlagFromStyle(fieldFlags.getVisibleFlag()))) {
                        newFlag = PomFactory.eINSTANCE.createVisibleFlag();
                        newFlag.setExpression((EqualityExpr)EcoreUtil.copy(spf.getVisibleExpression()));
                        fieldFlags.setVisibleFlag((VisibleFlag)newFlag);
                    } else if (spf.getEditableExpression() != null && (fieldFlags.getEditableFlag() == null || isFlagFromStyle(fieldFlags.getEditableFlag()))) {
                        newFlag = PomFactory.eINSTANCE.createEditableFlag();
                        newFlag.setExpression((EqualityExpr)EcoreUtil.copy(spf.getEditableExpression()));
                        fieldFlags.setEditableFlag((EditableFlag)newFlag);
                    } else if (spf.getCollapsedExpression() != null && (fieldFlags.getCollapsedFlag() == null || isFlagFromStyle(fieldFlags.getCollapsedFlag()))) {
                        newFlag = PomFactory.eINSTANCE.createCollapsedFlag();
                        newFlag.setExpression((EqualityExpr)EcoreUtil.copy(spf.getCollapsedExpression()));
                        fieldFlags.setCollapsedFlag((CollapsedFlag)newFlag);                                                               
                    }
                    if (newFlag != null) {
                        setFlagFromStyle(newFlag);
                    }
                }
            } else if (sp instanceof StylePropertyAlignment) {
                StylePropertyAlignment sap = (StylePropertyAlignment)sp;
                if (obj instanceof TableColumn) {
                    // TODO: replace the follwing 2 lines (before the modeling of colums as StyleSelectorType is needed)
//                    TableColumn bp = (TableColumn)obj;
//                    bp.setAlign(sap.getAlignment());
                }
            } else if (sp instanceof StylePropertyImage && obj instanceof XmadslPage) {
                StylePropertyImage spi = (StylePropertyImage)sp;
                XmadslPage xmadslPage = (XmadslPage)obj;
                ImageProperty imageProperty = PomFactory.eINSTANCE.createImageProperty();
                if (spi.getImageUri() != null) {
                    imageProperty.setImageUri(spi.getImageUri());
                } else if (spi.getImageUriReference() != null) {
                    imageProperty.setImageUriReference((ImageUri)EcoreUtil2.copy(spi.getImageUriReference()));
                }
                xmadslPage.getPageProperty().add(imageProperty);
            }
        }
        if (fieldFlags != null) {
            fieldFlags.updateSourceFieldFlagList();
        }
    }

    private static void determineDefaultStyleForModelElement(EObject obj) {
        if (obj instanceof IElementWithLayoutData) {
            String id = PomNameProvider.getXmaName(obj);
            String defaultStyleName = GeneratorConfiguration.INSTANCE.getLayoutStrategy().getDefaultStyle(obj,id);
            if (defaultStyleName != null) {
                Style defaultStyle    = CoreFactory.eINSTANCE.createStyle();
                defaultStyle.setName(defaultStyleName);
                IElementWithLayoutData elementWithLayoutData = (IElementWithLayoutData)obj;
                addDefaultStyle(elementWithLayoutData,defaultStyle);
            }
        }
    }

    public static boolean canModelElementHaveStyle(EObject obj) {
        if (obj instanceof IElementWithLayoutData) return true;
        return false;
    }
    public static List<Style> getStyles(EObject obj) {
        if (canModelElementHaveStyle(obj)) {
            IElementWithLayoutData elementWithLayoutData = (IElementWithLayoutData)obj;
            StyleProperty styleProp = new LayoutData(elementWithLayoutData).getStyleProperty();
            if (styleProp != null) {
                return styleProp.getStyles();
            }
        }
        if (obj instanceof XmadslPage) {
            XmadslPage p = (XmadslPage)obj;
            for (PageProperty pp: p.getPageProperty()) {
                if (pp instanceof StyleProperty) {
                    StyleProperty sp = (StyleProperty)pp;
                    return sp.getStyles();
                }
            }
        }
        if (obj instanceof Component) {
            Component c = (Component)obj;
            if (c.getStyleProperty() != null) {
                return c.getStyleProperty().getStyles();
            }
        }
        if (obj instanceof TableColumn) {
            TableColumn tc = (TableColumn)obj;
            if (tc.getStyleProperty() != null) {
                return tc.getStyleProperty().getStyles();
            }
        }
        if (obj instanceof TabPage) {
            TabPage tp = (TabPage)obj;
            if (tp.getStyleProperty() != null) {
                return tp.getStyleProperty().getStyles();
            }
        }
        return null;
    }

    /**
     * Add a default Style to the first position in the style list.
     * @param obj
     * @param style
     */
    public static void addDefaultStyle(EObject obj, Style style) {
        if (canModelElementHaveStyle(obj)) {
            IElementWithLayoutData elementWithLayoutData = (IElementWithLayoutData)obj;
            LayoutData layoutData = new LayoutData(elementWithLayoutData);
            layoutData.getStyles().add(0,style);
        }
    }

    public static List<Style> getStyleProperty(EObject obj) {
        Object value = EObjectPropertiesAdapter.getProperty(obj,STYLE_PROPERTY_NAME);
        if (value instanceof List<?> || value == null) {
            return (List<Style>)value;
        }
        throw new RuntimeException("The property '"+STYLE_PROPERTY_NAME+"' is not of type 'List<Style>'");
    }

    public static StylePropertyLineWrap getLineWrapStyle(Map<Integer,StyleSpecificationProperty> styleSpecificationProperties) {
        return (StylePropertyLineWrap)StyleAccess.getStyleProperty(styleSpecificationProperties,PomPackage.STYLE_PROPERTY_LINE_WRAP);
    }

    public static StylePropertyMultiLine getMultiLineStyle(Map<Integer,StyleSpecificationProperty> styleSpecificationProperties) {
        return (StylePropertyMultiLine)StyleAccess.getStyleProperty(styleSpecificationProperties,PomPackage.STYLE_PROPERTY_MULTI_LINE);
    }

    public static StylePropertyEnumerationDisplayType getEnumerationDisplayTypeStyle(Map<Integer,StyleSpecificationProperty> styleSpecificationProperties) {
        return (StylePropertyEnumerationDisplayType)StyleAccess.getStyleProperty(styleSpecificationProperties,PomPackage.STYLE_PROPERTY_ENUMERATION_DISPLAY_TYPE);
    }

    public static StylePropertyEnumerationSortOrder getEnumerationSortOrderStyle(Map<Integer,StyleSpecificationProperty> styleSpecificationProperties) {
        return (StylePropertyEnumerationSortOrder)StyleAccess.getStyleProperty(styleSpecificationProperties,PomPackage.STYLE_PROPERTY_ENUMERATION_SORT_ORDER);
    }

    public static StylePropertyAlignment getAlignmentStyle(Map<Integer,StyleSpecificationProperty> styleSpecificationProperties) {
        return (StylePropertyAlignment)StyleAccess.getStyleProperty(styleSpecificationProperties,PomPackage.STYLE_PROPERTY_ALIGNMENT);
    }

    public static StylePropertyOther getStylePropertyOther(Map<Integer, StyleSpecificationProperty> styleSpecificationProperties) {
        return (StylePropertyOther)StyleAccess.getStyleProperty(styleSpecificationProperties,PomPackage.STYLE_PROPERTY_OTHER);
    }
    
    public static StylePropertyFlag getStylePropertyFlag(Map<Integer, StyleSpecificationProperty> styleSpecificationProperties) {
        return (StylePropertyFlag)StyleAccess.getStyleProperty(styleSpecificationProperties,PomPackage.STYLE_PROPERTY_FLAG);
    }

    public static StylePropertyWidgetVariant getStylePropertyWidgetVariant(Map<Integer, StyleSpecificationProperty> styleSpecificationProperties) {
        return (StylePropertyWidgetVariant)StyleAccess.getStyleProperty(styleSpecificationProperties,PomPackage.STYLE_PROPERTY_WIDGET_VARIANT);
    }
    
    public static StylePropertyForegroundColor getStylePropertyForegroundColor(Map<Integer, StyleSpecificationProperty> styleSpecificationProperties) {
        return (StylePropertyForegroundColor)StyleAccess.getStyleProperty(styleSpecificationProperties,PomPackage.STYLE_PROPERTY_FOREGROUND_COLOR);
    }
    
    public static StylePropertyBackgroundColor getStylePropertyBackgroundColor(Map<Integer, StyleSpecificationProperty> styleSpecificationProperties) {
        return (StylePropertyBackgroundColor)StyleAccess.getStyleProperty(styleSpecificationProperties,PomPackage.STYLE_PROPERTY_BACKGROUND_COLOR);
    }
    
    public static StylePropertyFont getStylePropertyFont(Map<Integer, StyleSpecificationProperty> styleSpecificationProperties) {
        return (StylePropertyFont)StyleAccess.getStyleProperty(styleSpecificationProperties,PomPackage.STYLE_PROPERTY_FONT);
    }      
    
    
        

}
